package require taotk
package require img::tiff
package require ctext
package require sqlite3

###
# Define classes
###

tao::class ubercanvas {
  superclass taotk::frame taotk::meta::canvas tao::onion

  option filename {}

  signal startup {
    triggers {content resize}
    action {
      my activate_layers
      my widget controls
    }
  }
  signal content {
    follows startup
    action {my widget content}
  }
  signal resize {
    follows content
    action {my widget resize}
  }

  property shared_organs canvas

  method action::fwd {} {
    global logstep logdat logturn
    while 1 {
      set move [string trim $::logdat([incr logstep])]
      if {[string length $move] eq 0} return
      if {[string index $move 0] eq "#"} return
      if {[lindex $move 0] eq "turn"} {
        set logturn [lindex $move 1]
        continue
      }
      if {[my layer players input $move]} {
        break
      }
    }
  }

  method action::back {} {
    
  }

  ###
  # description:
  # To allow random access, we need to load the
  # data into temporary storage. This function populates
  # and array with line number as the key
  ###
  method Option_set::filename newvalue {
    if {$newvalue eq {}} {
      return
    }
    global logdat logstep
    array unset logdat *
    set fin [open $newvalue r]
    set linenum 0
    while {[gets $fin line]>=0} {
      set logdat([incr linenum]) $line
    }
    close $fin
    set logstep 0
    my signal startup
  }
  
  method active_layers {} {
    return {
      map {class uberlayer.map order 1 prefix m}
      players {class uberlayer.players order 2 prefix p}
    }
  }
  
  method widget::components {} {
    set hull [my widget hull]
    ttk::frame $hull.controls
    my graft controlframe $hull.controls
    pack $hull.controls -side top -fill x
    my Build_canvas $hull
  }
  
  method widget::controls {} {
    set frame [my organ controlframe]
    destroy {*}[winfo children $frame]
    button $frame.load -text "Load" -command [namespace code {my configure filename [tk_getOpenFile]}]
    
    button $frame.back -text "-" -command  [namespace code {my action back}]
    button $frame.fwd -text "+" -command   [namespace code {my action fwd}]
    button $frame.run -text "run" -command [namespace code {my action run}]
    entry $frame.step -textvariable logstep 
    button $frame.goto -text "goto" -command [namespace code {my action goto}]
    label $frame.turn#l -text "Turn:"
    label $frame.turn -textvariable logturn
    
    pack $frame.load $frame.back $frame.fwd $frame.run -side left -fill y
    pack $frame.step $frame.goto $frame.turn $frame.turn#l -side right -fill y

  }
  
  method widget::content {} {
    my canvas delete all
    foreach layer {map players} {
      my layer $layer layer_redraw
    }
  }
  
  method widget::resize {} {
    set sizerq [my widget reqsize]
    lassign $sizerq sizex sizey dyn
    wm geometry . ${sizex}x${sizey}
  }
}

tao::class uberlayer {
  superclass tao::layer
  
  method layer_redraw {} {}
  method layer_update {} {}
}

tao::class uberlayer.map {
  superclass uberlayer
  
  method initialize {} {
    global thiscript
    image create photo map -file [file join $thiscript .. WorldMap.tiff]
  }
  
  method layer_redraw {} {
    my canvas create image 0 0 -image map -anchor nw
  }
  
  method layer_update {} {
    
  }
}

tao::class uberlayer.players {
  superclass uberlayer
  
  signal update {
    follows content
    action {my layer_update}
  }
  
  method initialize {} {
    my variable playercount
    set playercount 0
    
    sqlite3 [namespace current]::db :memory:
    my graft db [namespace current]::db
    # Build the db
    my db eval {
create table player (
  player string primary key,
  color string
);
create table region (
  regionid integer primary key,
  x real,
  y real,
  owner string references player,
  armies integer default 0,
  redraw boolean default 0
);
    }
    
    ###
    # Index
    global thiscript
    set coordsfile  [file join $thiscript coords.csv]
    if {![file exists $coordsfile]} {
      exec [info nameofexecutable] [file join $here capture.tcl]
    }
    set fin [open $coordsfile r]
    while {[gets $fin line] >= 0 } {
      lassign [split $line ,] region x y
      my db eval {insert into region (regionid,x,y) VALUES (:region,:x,:y);}
    }
    close $fin
    
  }
  
  method input move {
    my variable playercount
    set player [lindex $move 0]
    if {![my db exists {select color from player where player=:player}]} {
      set color [lindex {red blue purple} [incr playercount]]
      my db eval {insert into player(player,color) values (:player,:color);}
    } else {
      set color [my db one {select color from player where player=:player}]
    }
    puts $move
    set update 0
    switch -- [lindex $move 1] {
      view {
      }
      {+=} {
      }
      place {
        set region [lindex $move 2]
        set count [lindex $move end]
        set owner [my db one {select owner from region where regionid=:region}]
        if {$owner eq $player} {
          set armies [my db one {select armies from region where regionid=:region}]
        } else {
          set armies 0
        }
        incr armies $count
        my db one {update region set owner=:player,armies=:armies,redraw=1 where regionid=:region}
        set update 1
      }
      attack {
        
      }
      versus {
        
      }
      fight {
        
      }
      kills {
        
      }
      left {
        
      }
      success {
        
      }
      failure {
        
      }
      move {
        
      }
    }
    my signal update
    return $update
  }
  method layer_redraw {} {
    my db eval {
      select regionid,color,x,y,armies from region,player where player.player=region.owner 
    } {
      set radius [expr {$armies*5}]
      if {$radius<=0} continue
      set x0 [expr {$x-$radius}]
      set y0 [expr {$y-$radius}]
      set x1 [expr {$x+$radius}]
      set y1 [expr {$y+$radius}]
      my canvas create oval $x0 $y0 $x1 $y1 -fill $color -width 1 -tags [list $player region#$regionid]
    }
    my db eval {update region set redraw=0}
  }
  method layer_update {} {
    my db eval {
      select player.player,regionid,color,x,y,armies from region,player where player.player=region.owner and redraw=1
    } {
      my canvas delete region#$regionid
      set radius [expr {$armies*5}]
      if {$radius<=0} continue
      set x0 [expr {$x-$radius}]
      set y0 [expr {$y-$radius}]
      set x1 [expr {$x+$radius}]
      set y1 [expr {$y+$radius}]
      my canvas create oval $x0 $y0 $x1 $y1 -fill $color -width 1 -tags [list $player region#$regionid]
    }
    my db eval {update region set redraw=0}
  }
}


###
# Load the coords from the file
###
set ::thiscript [file dirname [file normalize [info script]]]
###
# Create our master object
###
ubercanvas create appmain .risk

pack .risk -side top -fill both -expand 1

appmain signal startup
console:start
